from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group as AuthGroup
from django.core.exceptions import PermissionDenied




class ACLBaseBackend:
    
    def get_user_permissions(self, user_obj, obj=None):
        return set()
    
    def get_role_permissions(self, roles):
        permissions = set()
        for role in roles:
            role_permissions = role.permissions.all()
            for role_permission in role_permissions:
                codename = f"auth.{role_permission.codename}"
                permissions.add(codename)
        return permissions
    

    def get_group_permissions(self, user_obj, obj=None):
        user_groups_field = get_user_model()._meta.get_field("user_groups")
        user_groups_query = "group__%s" % user_groups_field.related_query_name()
        
        roles = AuthGroup.objects.filter(**{user_groups_query: user_obj})
        permissions = self.get_role_permissions(roles)
        
        return set(permissions)
    

    def get_all_permissions(self, user_obj, obj=None):
        return {
            *self.get_user_permissions(user_obj, obj=obj),
            *self.get_group_permissions(user_obj, obj=obj),
        }

    def has_acl_perm(self, user_obj, perm, obj=None):
        return perm in self.get_all_permissions(user_obj, obj=obj)
    
    


def acl_has_perms(user, perm, obj):
    
    """
    A backend can raise `PermissionDenied` to short-circuit permission checking.
    """
    try:
        if ACLBaseBackend().has_acl_perm(user_obj=user, perm=perm, obj=obj) or user.is_superuser:
            return True
    except PermissionDenied:
        return False
    return False


from django.contrib import admin
from django.contrib.contenttypes.models import ContentType

from django.contrib.auth.models import Permission
from django.contrib.auth.admin import GroupAdmin as AuthRoleAdmin
from django.contrib.auth.models import Group as AuthRole

from .models import Group, Role


class PermissionsAdmin(admin.ModelAdmin):
    model = Permission
    list_display = ['id', 'name','content_type','codename','label']

admin.site.register(Permission,PermissionsAdmin)



class ContentTypeAdmin(admin.ModelAdmin):
    model = ContentType
    list_display = ['id', 'app_label','model',]
    
admin.site.register(ContentType,ContentTypeAdmin)






class RoleAdmin(AuthRoleAdmin):
    list_display = ['name','get_permissions']
    
    def get_permissions(self,obj):
        return [obj.codename for obj in obj.permissions.all()]


admin.site.unregister(AuthRole)
admin.site.register(Role, RoleAdmin)




@admin.register(Group)
class GroupAdmin(admin.ModelAdmin):
    list_display = ('name','get_roles')
    search_fields = ('name',)
    


from django.core.management.base import BaseCommand
from django.contrib.contenttypes.models import ContentType
from django.core.management import call_command
from django.db.models import Q
from django.contrib.auth import get_user_model

from django.contrib.auth.models import Permission

class Command(BaseCommand):
    help = 'Closes the specified poll for voting'


    def handle(self, *args, **options):
        
        call_command('makemigrations')
        call_command('migrate')
        
        
        if get_user_model().objects.filter(is_superuser=1).count() == 0:
            call_command('createsuperuser')
            
            
        content_type = ContentType.objects.get(model='role')
        
        Permission.objects.filter(~Q(content_type=content_type)).delete()
        
        
        


from django.db import models
from django.contrib.auth.models import Permission
from django.contrib.auth.models import Group as AuthRole
from django.utils.translation import gettext_lazy as _


class Role(AuthRole):
    class Meta:
        proxy = True
        app_label = 'auth'
        verbose_name = _('Role')
        default_permissions  = ()
        
        
        
class GroupManager(models.Manager):
    use_in_migrations = True

    def get_by_natural_key(self, name):
        return self.get(name=name)



class Group(models.Model):
    name = models.CharField(_("name"), max_length=150, unique=True)
    roles = models.ManyToManyField(
        AuthRole,
        verbose_name=_("roles"),
        blank=True,
    )

    objects = GroupManager()

    class Meta:
        verbose_name = _("group")
        verbose_name_plural = _("groups")
        
    def get_roles(self):
        return ",".join([str(p) for p in self.roles.all()])

    def __str__(self):
        return self.name

    def natural_key(self):
        return (self.name,)






if not hasattr(Permission, 'label'):
    label = models.CharField(db_column='label', max_length=255, blank=True, null=False)
    label.contribute_to_class(Permission, 'label')
    
    
    